package cz.jzitnik.tui;

import lombok.extern.slf4j.Slf4j;

import javax.sound.sampled.*;
import java.io.IOException;

@Slf4j
public class SoundPlayer {
    public static void playSound(String filePath)
            throws LineUnavailableException, IOException, UnsupportedAudioFileException, InterruptedException {
        if (!filePath.endsWith(".ogg")) {
            return;
        }

        log.info("Loading resource: {}", "sounds/" + filePath);

        var file = SoundPlayer.class.getClassLoader().getResourceAsStream("sounds/" + filePath);
        AudioInputStream audioStream = AudioSystem.getAudioInputStream(file);
        AudioFormat baseFormat = audioStream.getFormat();

        // Convert the audio format to PCM_SIGNED
        AudioFormat targetFormat = new AudioFormat(
                AudioFormat.Encoding.PCM_SIGNED,
                baseFormat.getSampleRate(),
                16,
                baseFormat.getChannels(),
                baseFormat.getChannels() * 2,
                baseFormat.getSampleRate(),
                false
        );

        // Apply the format change to the stream
        AudioInputStream dataIn = AudioSystem.getAudioInputStream(targetFormat, audioStream);

        byte[] buffer = new byte[8192]; // Larger buffer to reduce read/write operations
        DataLine.Info info = new DataLine.Info(SourceDataLine.class, targetFormat);

        try (SourceDataLine line = (SourceDataLine) AudioSystem.getLine(info)) {
            if (line != null) {
                line.open(targetFormat);
                line.start();

                int bytesRead;
                while ((bytesRead = dataIn.read(buffer, 0, buffer.length)) != -1) {
                    line.write(buffer, 0, bytesRead);
                }

                // Ensure the line is fully flushed before stopping
                line.drain();
            }
        }

        // Close streams after use
        dataIn.close();
        audioStream.close();
    }
}
